#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# glite-rgma-server-config v. 5.2.4
#
# Post-installation script for configuring the gLite R-GMA server
# Joachim Flammer <Joachim.Flammer@cern.ch>
#
# Version info: $Id: glite-rgma-server-config.py,v 1.39.2.17 2006/05/02 10:32:11 flammer Exp $
#
# Usage: python glite-rgma-server-config [-c|-v|-h|--help]
#        -c, --checkconf     print configuration
#        -v, --version       print version
#        -h, --help          print usage info
#        --verbose           turn verbosity on
#
#        --configure         configure service
#        --start             start service
#        --stop              stop service
#        --status            print service status (0 = running, 1 = idle)
#
#        --recreate_db       force recreation of DB for configuration (additional option for --configure)
#        --siteconfig=URL    specify URL of the site configuration file
#
# Return codes: 0 - Ok
#               1 - Failed
#
#####################################################################################

# import system libraries
import os,string,socket,pwd
import sys
import getopt
import time
import commands

from gLiteInstallerLib import gLib
from gLiteInstallerLib import ConfigParams

from mysql import Mysql
from tomcat5 import Secure_tomcat5 as Tomcat5
from gliteRgmaServicetool import gliteRgmaServicetool

# Set global variables here 
global params                     # all config values from the XML file
global rgmaServicetool

class glite_rgma_server:

    #################################################################
    # init
    #
    # initialize class
    #
    #################################################################
    def __init__(self):
        self.verbose = 0
        self.version = "5.2.4"
        self.name = "glite-rgma-server"
        self.friendly_name = "gLite R-GMA Server"
        

    #################################################################
    # banner
    #
    # print banner
    #
    #################################################################
    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    

    #################################################################
    # copyright
    #
    # print copyright
    #
    #################################################################
    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'


    #################################################################
    # version
    #
    # print version
    #
    #################################################################
    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
       
    
    #################################################################
    # usage 
    #
    # print usage
    #
    #################################################################
    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()
    
        print "Usage: \n"
        print "Edit the configuration file %s.cfg.xml in " %  self.name,
        print "%s/etc.config/templates" % os.environ['GLITE_LOCATION']
        print "save it as %s/etc/config/%s.cfg.xml" % (os.environ['GLITE_LOCATION'], self.name),
        print "and run the script as follows"
        print "python %s-config [OPTION...]" % self.name
        print ''
        print '    -c, --checkconf     print the service configuration'
        print '    -v, --version       print the version of the configuration script'
        print '    -h, --help          print this usage information'
        print '    --verbose           turn verbosity on'
        print
        print '    --configure         configure %s' % self.friendly_name
        print '    --start             start %s' % self.friendly_name
        print '    --stop              stop %s' % self.friendly_name
        print '    --status            print service status (0 = running, 1 = idle)'
        print ''
        print '    --recreate_db       force recreation of DB for configuration'
	print '                        to be specified with option --configure'
        print '    --siteconfig=URL    URL of the site configuration file'
        print ''
        print '\n'


    #################################################################
    # configure
    #
    # configure rgma server
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configure(self):        
            
        # ------------------------------------------------------------        
        # Check certificates
        # ------------------------------------------------------------    

        if params.has_key('glite.installer.checkcerts'):
            if params['glite.installer.checkcerts'] == "true":                
                if glib.check_certs(params) != 0:
                    return 1
                    
        # ------------------------------------------------------------
        # Run security util script
        # ------------------------------------------------------------

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "              Configuring the gLite security utils"   
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
            
        errorcode = os.system("%s/etc/config/scripts/glite-security-utils-config.py --subservice" % os.environ['GLITE_LOCATION'])

        if errorcode:
            print "Configuring gLite security                                ",
            glib.printFailedMessage()
            return 1
        
        self.banner()
                    
        # ------------------------------------------------------------
        # get corresponding configuration values
        # ------------------------------------------------------------
                
        print "Checking configuration values ...                         "

        # ------------------------------------------------------------
        #  R-GMA general --- get corresponding configuration values
        # ------------------------------------------------------------

        # ---user defined ---
        
        # rgma server hostname (user)
        if params.has_key('rgma.server.hostname'):
            self.server_hostname = params['rgma.server.hostname']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.server.hostname\" not found.")
            return 1
        
        # rgma schema hostname (user)
        if params.has_key('rgma.schema.hostname'):
            self.schema_hostname = params['rgma.schema.hostname']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.schema.hostname\" not found.")
            return 1                        
        
        # rgma registry hostname: at least one must be given (user)
        if not len(params['rgma.registry.hostname']) > 0:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.registry.hostname\" not found.")
            return 1

        # --- advanced ---

        # host certificate file (advanced - global)        
        self.host_certificate_file = params['host.certificate.file']

        # host.key.file (advanced - global)
        self.host_key_file = params['host.key.file']

        # ca.certificates.dir (advanced - global)
        self.ca_certificates_dir = params['ca.certificates.dir']
                
        # user certificates path (advanced - global)
        self.user_certificates_path = params['user.certificate.path']        

        # rgma secure mode (advanced - rgma common)
        self.rgma_secure_mode = params['rgma.secure.mode']
       
        # --- system ---
        
        # rgma user name (system - rgma common)
        self.rgma_user_name = params['rgma.user.name']
            
        # rgma user group (system - rgma common)
        self.rgma_user_group = params['rgma.user.group']

        # ------------------------------------------------------------
        #  values for server --- get corresponding configuration values
        # ------------------------------------------------------------

        # --- user defined ---
        
        # MySQL root password (user)
        if params.has_key('rgma.server.mysql_root_password'):
            self.mysql_root_password = params['rgma.server.mysql_root_password']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.server.mysql_root_password\" not found.")
            return 1
 
        # run schema_service (user)
        if params.has_key('rgma.server.run_schema_service'):
            
            # ensure backwards compatability:
            if params['rgma.server.run_schema_service'] == "yes":
                glib.printWarningMessage("Parameter 'rgma.server.run_schema_service' is set to yes.")
                glib.printWarningMessage("Parameter has changed to a boolean parameter (true|false) and is no longer (yes|no).") 
                glib.printWarningMessage("Please change the parameter to 'true|false'.")       
                glib.printWarningMessage("To ensure backwards compatability, the parameter is taken as 'true' for the configuration.")
                
                self.run_schema_service = "yes"
            
            # ensure backwards compatability:    
            elif params['rgma.server.run_schema_service'] == "no":
                glib.printWarningMessage("Parameter 'rgma.server.run_schema_service' is set to no.")
                glib.printWarningMessage("Parameter has changed to a boolean parameter (true|false) and is no longer (yes|no).") 
                glib.printWarningMessage("Please change the parameter to 'true|false'.")       
                glib.printWarningMessage("To ensure backwards compatability, the parameter is taken as 'false' for the configuration.")
                
                self.run_schema_service = "no"
            
            elif params['rgma.server.run_schema_service'] == "true":
                
                self.run_schema_service = "yes"
                
            elif params['rgma.server.run_schema_service'] == "false":
                
                self.run_schema_service = "no"
                
            else:
                glib.printErrorMessage("Parameter 'rgma.server.run_schema_service' specified incorrectly.")
                glib.printErrorMessage("Must be 'true|false' while '%s' has been specified." % params['rgma.server.run_schema_service'])
                glib.printErrorMessage("Correct error and re-run configuration.")
                return 1
                
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.server.run_schema_service\" not found.")
            return 1
            
        # run registry_service (user)
        if params.has_key('rgma.server.run_registry_service'):
            
            # ensure backwards compatability:
            if params['rgma.server.run_registry_service'] == "yes":
                glib.printWarningMessage("Parameter 'rgma.server.run_registry_service' is set to yes.")
                glib.printWarningMessage("Parameter has changed to a boolean parameter (true|false) and is no longer (yes|no).") 
                glib.printWarningMessage("Please change the parameter to 'true|false'.")       
                glib.printWarningMessage("To ensure backwards compatability, the parameter is taken as 'true' for the configuration.")
                
                self.run_registry_service = "yes"
            
            # ensure backwards compatability:    
            elif params['rgma.server.run_registry_service'] == "no":
                glib.printWarningMessage("Parameter 'rgma.server.run_registry_service' is set to no.")
                glib.printWarningMessage("Parameter has changed to a boolean parameter (true|false) and is no longer (yes|no).") 
                glib.printWarningMessage("Please change the parameter to 'true|false'.")       
                glib.printWarningMessage("To ensure backwards compatability, the parameter is taken as 'false' for the configuration.")
                
                self.run_registry_service = "no"
            
            elif params['rgma.server.run_registry_service'] == "true":
                
                self.run_registry_service = "yes"
                
            elif params['rgma.server.run_registry_service'] == "false":
                
                self.run_registry_service = "no"
                
            else:
                glib.printErrorMessage("Parameter 'rgma.server.run_registry_service' specified incorrectly.")
                glib.printErrorMessage("Must be 'true|false' while '%s' has been specified." % params['rgma.server.run_registry_service'])
                glib.printErrorMessage("Correct error and re-run configuration.")
                return 1
                
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.server.run_registry_service\" not found.")
            return 1
            
        # run browser (user)
        if params.has_key('rgma.server.run_browser'):
            
            # ensure backwards compatability:
            if params['rgma.server.run_browser'] == "yes":
                glib.printWarningMessage("Parameter 'rgma.server.run_browser' is set to yes.")
                glib.printWarningMessage("Parameter has changed to a boolean parameter (true|false) and is no longer (yes|no).") 
                glib.printWarningMessage("Please change the parameter to 'true|false'.")       
                glib.printWarningMessage("To ensure backwards compatability, the parameter is taken as 'true' for the configuration.")
                
                self.run_browser = "yes"
            
            # ensure backwards compatability:    
            elif params['rgma.server.run_browser'] == "no":
                glib.printWarningMessage("Parameter 'rgma.server.run_browser' is set to no.")
                glib.printWarningMessage("Parameter has changed to a boolean parameter (true|false) and is no longer (yes|no).") 
                glib.printWarningMessage("Please change the parameter to 'true|false'.")       
                glib.printWarningMessage("To ensure backwards compatability, the parameter is taken as 'false' for the configuration.")
                
                self.run_browser = "no"
            
            elif params['rgma.server.run_browser'] == "true":
                
                self.run_browser = "yes"
                
            elif params['rgma.server.run_browser'] == "false":
                
                self.run_browser = "no"
                
            else:
                glib.printErrorMessage("Parameter 'rgma.server.run_browser' specified incorrectly.")
                glib.printErrorMessage("Must be 'true|false' while '%s' has been specified." % params['rgma.server.run_browser'])
                glib.printErrorMessage("Correct error and re-run configuration.")
                return 1
                
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.server.run_browser\" not found.")
            return 1                      
            
        # --- advanced ---

        # rgma max threads (advanced)
        self.httpconnector_maxthreads = params['rgma.server.httpconnector_maxthread']
        
        # rgma max threads (advanced)
        self.httpconnector_enableLookups = params['rgma.server.httpconnector_enableLookups']
        
        # rgma max threads (advanced)
        self.httpconnector_maxPostSize = params['rgma.server.httpconnector_maxPostSize']

        # LD_ASSUME_KERNEL (advanced)
        self.server_ldAssumeKernel = params['rgma.server.LD_ASSUME_KERNEL']
        
        # allow unsecure port (advanced)
        self.allow_unsecure_port = params['allow.unsecure.port']

        # force recreation of DB (advanced)
        self.recreate_db = params['rgma.server.recreate_db']
        
        # --- system ---
            
        # rgma webapplication deploy path (system)
        self.rgma_webapp_path = params['rgma.server.webapp_path']
            
        # rgma webapplication war name (system)
        self.rgma_war_name = params['rgma.server.war_name']
                        
        # tomcat user name (system - global)
        self.tomcat_user_name = params['tomcat.user.name']
            
        # tomcat user group (system - global)
        self.tomcat_user_group = params['tomcat.user.group']
      
        # server security configuration file (system)
        self.server_security_config_file = params['rgma.server.security.configurationFile']
        
        # specific rgma server tomcat catalina options
        self.rgma_server_tomcat_catalina_opts = params['rgma.server.tomcat.CATALINA_OPTS']
            
        # ------------------------------------------------------------
        #  values for glue-archiver --- get corresponding configuration values
        # ------------------------------------------------------------

        # --- user defined ---

        # rgma run archiver (user)
        if params.has_key('rgma.server.run_archiver'):
            
            # ensure backwards compatability:
            if params['rgma.server.run_archiver'] == "yes":
                glib.printWarningMessage("Parameter 'rgma.server.run_archiver' is set to yes.")
                glib.printWarningMessage("Parameter has changed to a boolean parameter (true|false) and is no longer (yes|no).") 
                glib.printWarningMessage("Please change the parameter to 'true|false'.")       
                glib.printWarningMessage("To ensure backwards compatability, the parameter is taken as 'true' for the configuration.")
                
                self.run_archiver = "yes"
            
            # ensure backwards compatability:    
            elif params['rgma.server.run_archiver'] == "no":
                glib.printWarningMessage("Parameter 'rgma.server.run_archiver' is set to no.")
                glib.printWarningMessage("Parameter has changed to a boolean parameter (true|false) and is no longer (yes|no).") 
                glib.printWarningMessage("Please change the parameter to 'true|false'.")       
                glib.printWarningMessage("To ensure backwards compatability, the parameter is taken as 'false' for the configuration.")
                
                self.run_archiver = "no"
            
            elif params['rgma.server.run_archiver'] == "true":
                
                self.run_archiver = "yes"
                
            elif params['rgma.server.run_archiver'] == "false":
                
                self.run_archiver = "no"
                
            else:
                glib.printErrorMessage("Parameter 'rgma.server.run_archiver' specified incorrectly.")
                glib.printErrorMessage("Must be 'true|false' while '%s' has been specified." % params['rgma.server.run_archiver'])
                glib.printErrorMessage("Correct error and re-run configuration.")
                return 1
                
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.server.run_archiver\" not found.")
            return 1

        if self.run_archiver == "yes":
        
            # --- advanced ---

            # rgma archiver db name (advanced)
            self.archiver_db_name = params['rgma.archiver.db.name']

            # rgma archiver db user (advanced)
            self.archiver_db_user = params['rgma.archiver.db.user']

            # rgma archiver db user (advanced)
            self.archiver_db_password = params['rgma.archiver.db.password']

            # rgma archiver db retention period (advanced)
            self.archiver_historyRetentionPeriod = params['rgma.archiver.db.historyRetentionPeriod']
        
            # --- system ---
        
            # rgma archiver configuration file (system)
            self.archiver_config_file = params['rgma.archiver.configurationFile']
       
       
        # ------------------------------------------------------------
        #  values for site-publisher --- get corresponding configuration values
        # ------------------------------------------------------------
        
        # --- user defined ---

        # rgma run site-publisher (user)
        if params.has_key('rgma.server.run_site-publisher'):
            
            # ensure backwards compatability:
            if params['rgma.server.run_site-publisher'] == "yes":
                glib.printWarningMessage("Parameter 'rgma.server.run_site-publisher' is set to yes.")
                glib.printWarningMessage("Parameter has changed to a boolean parameter (true|false) and is no longer (yes|no).") 
                glib.printWarningMessage("Please change the parameter to 'true|false'.")       
                glib.printWarningMessage("To ensure backwards compatability, the parameter is taken as 'true' for the configuration.")
                
                self.run_site_publisher = "yes"
            
            # ensure backwards compatability:    
            elif params['rgma.server.run_site-publisher'] == "no":
                glib.printWarningMessage("Parameter 'rgma.server.run_site-publisher' is set to no.")
                glib.printWarningMessage("Parameter has changed to a boolean parameter (true|false) and is no longer (yes|no).") 
                glib.printWarningMessage("Please change the parameter to 'true|false'.")       
                glib.printWarningMessage("To ensure backwards compatability, the parameter is taken as 'false' for the configuration.")
                
                self.run_site_publisher = "no"
            
            elif params['rgma.server.run_site-publisher'] == "true":
                
                self.run_site_publisher = "yes"
                
            elif params['rgma.server.run_site-publisher'] == "false":
                
                self.run_site_publisher = "no"
                
            else:
                glib.printErrorMessage("Parameter 'rgma.server.run_site-publisher' specified incorrectly.")
                glib.printErrorMessage("Must be 'true|false' while '%s' has been specified." % params['rgma.server.run_site-publisher'])
                glib.printErrorMessage("Correct error and re-run configuration.")
                return 1
                
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.server.run_site-publisher\" not found.")
            return 1

        # get site-publisher configuration values
        if self.run_site_publisher == "yes":
        
            # sysadmin contact (user)
            if params.has_key('rgma.site-publisher.contact.system_administrator'):
                self.sitePublisher_sysAdminContact = params['rgma.site-publisher.contact.system_administrator']
            else:
                glib.printFailedMessage()
                glib.printErrorMessage("Parameter \"rgma.site-publisher.contact.system_administrator\" not found. Please correct configuration file.")
                return 1

            # user support contact (user)
            if params.has_key('rgma.site-publisher.contact.user_support'):
                self.sitePublisher_userSupportContact = params['rgma.site-publisher.contact.user_support']
            else:
                glib.printFailedMessage()
                glib.printErrorMessage("Parameter \"rgma.site-publisher.contact.user_support\" not found. Please correct configuration file.")
                return 1
            
            # security contact (user)
            if params.has_key('rgma.site-publisher.contact.site_security'):
                self.sitePublisher_securityContact = params['rgma.site-publisher.contact.site_security']
            else:
                glib.printFailedMessage()
                glib.printErrorMessage("Parameter \"rgma.site-publisher.contact.site_security\" not found. Please correct configuration file.")
                return 1
            
            # location latitude (user)
            if params.has_key('rgma.site-publisher.location.latitude'):
                self.sitePublisher_locationLatitude = params['rgma.site-publisher.location.latitude']
            else:
                glib.printFailedMessage()
                glib.printErrorMessage("Parameter \"rgma.site-publisher.location.latitude\" not found. Please correct configuration file.")
                return 1

            # location longitude (user)
            if params.has_key('rgma.site-publisher.location.longitude'):
                self.sitePublisher_locationLongitude = params['rgma.site-publisher.location.longitude']
            else:
                glib.printFailedMessage()
                glib.printErrorMessage("Parameter \"rgma.site-publisher.location.longitude\" not found. Please correct configuration file.")
                return 1
                
            # location (user)
            if params.has_key('rgma.site-publisher.location'):
                self.sitePublisher_location = params['rgma.site-publisher.location']
            else:
                glib.printFailedMessage()
                glib.printErrorMessage("Parameter \"rgma.site-publisher.location\" not found. Please correct configuration file.")
                return 1
          
            # human readable description (user)
            if params.has_key('rgma.site-publisher.description'):
                self.sitePublisher_description = params['rgma.site-publisher.description']
            else:
                glib.printFailedMessage()
                glib.printErrorMessage("Parameter \"rgma.site-publisher.description\" not found. Please correct configuration file.")
                return 1
            
            # human readable site name (user)
            if params.has_key('rgma.site-publisher.siteName'):
                self.sitePublisher_siteName = params['rgma.site-publisher.siteName']
            else:
                glib.printFailedMessage()
                glib.printErrorMessage("Parameter \"rgma.site-publisher.siteName\" not found. Please correct configuration file.")
                return 1            

            # web location of site (user)
            if params.has_key('rgma.site-publisher.webLocation'):
                self.sitePublisher_webLocation = params['rgma.site-publisher.webLocation']
            else:
                glib.printFailedMessage()
                glib.printErrorMessage("Parameter \"rgma.site-publisher.webLocation\" not found. Please correct configuration file.")
                return 1             

            # --- advanced ---

            # site name (advanced)
            self.sitePublisher_siteId = params['rgma.site-publisher.siteId']
         
            # -- system ---
        
            # site publisher configuration file (system)
            self.sitePublisher_configFile = params['rgma.site-publisher.configurationFile']
 
        print "\nChecking configuration values                             ",         
        glib.printOkMessage()
        
        # temporary message as long as only one registry is allowed
        if len(params['rgma.registry.hostname']) > 1:
            print
            glib.printWarningMessage("You specified more than one registry.")
            glib.printWarningMessage("At the moment only one registry is allowed.")
            glib.printWarningMessage("Configuration will use first specified registry '%s'" % params['rgma.registry.hostname'][0])
            glib.printWarningMessage("and ignore the others.")
            print
                   
        #------------------------------------------------------------
        # Verify the JAVA installation and set it in /etc/java.conf
        #------------------------------------------------------------
            
        if glib.checkJava() != 0:
            return 1
                                
        # ------------------------------------------------------------
        # setup tomcat environment
        # ------------------------------------------------------------
        
        if self.configureTomcatWebapp():
            return 1
 
        # ------------------------------------------------------------
        # Start MySQL (if not yet running)
        # ------------------------------------------------------------

        if mysql.start():
	    return 1

        # ------------------------------------------------------------
        # Check configuration of MySQL
        # ------------------------------------------------------------
        	
        if not params.has_key('set.mysql.root.password'):
            params['set.mysql.root.password'] = 'false'
        setempty = params['set.mysql.root.password']
        if mysql.checkMySQLConfiguration(self.mysql_root_password,setempty):
            return 1

        
        # ------------------------------------------------------------
        # configure R-GMA common things
        # ------------------------------------------------------------

        if self.configureRgma():
            return 1
        
        # ------------------------------------------------------------
        # configure R-GMA server
        # ------------------------------------------------------------

        if self.configureRgmaServer():
            return 1
            
        # ------------------------------------------------------------
        # configure R-GMA server security
        # ------------------------------------------------------------

        if self.configureRgmaServerSecurity():
            return 1

        # ------------------------------------------------------------
        # if site-publisher option is turned on -or-
        # if glue-archiver option is turned on : 
        #                      configure R-GMA client
        # ------------------------------------------------------------
        if (self.run_site_publisher=="yes" or self.run_archiver=="yes"):
            
            # ------------------------------------------------------------
            # configure R-GMA client
            # ------------------------------------------------------------        
        
            if self.configureRgmaClient():
                return 1

            if self.configureRgmaClientSecurity():
	        return 1
            
        # ------------------------------------------------------------
        # if site-publisher option is turned on: configure site-publisher and related things
        # ------------------------------------------------------------
        if self.run_site_publisher=="yes":        

            if self.configureSitePublisher():
                return 1

        # ------------------------------------------------------------
        # if archiver option is turned on: configure archiver and related things
        # ------------------------------------------------------------
        if self.run_archiver=="yes":
            
            if self.configureFlexibleArchiver():
                return 1
                                
        #-------------------------------------------------------------------
        # R-GMA servicetool: configure servicetool and add subservices
        #-------------------------------------------------------------------
        print
        if rgmaServicetool.configure(glib):
            # error in configuring services
            return 1 

        # ------------------------------------------------------------
        # temporary bug fix for bug 9375 (no creation of log file)
        # ------------------------------------------------------------
        rgma_server_log_file = "rgma-server.log"
        rgma_server_log_dir  = "%s/rgma-server" % os.environ['GLITE_LOCATION_LOG']
        
        if not os.path.exists("%s/%s" % (rgma_server_log_dir, rgma_server_log_file)):
            
            # create dir if necessary
            if not os.path.exists("%s" % rgma_server_log_dir):                
                os.mkdir("%s" % rgma_server_log_dir, 755)
            
            # create log file
            output = open("%s/%s" % (rgma_server_log_dir, rgma_server_log_file), 'w')
            output.close()

            # change owner
            tomcat_ids = glib.get_user_info(self.tomcat_user_name)            
            os.chown("%s/%s" % (rgma_server_log_dir, rgma_server_log_file), 
                tomcat_ids[0], tomcat_ids[1])

        # ------------------------------------------------------------
        # registering R-GMA server
        # ------------------------------------------------------------
        
        glib.registerService()
            
        return 0
        

    #################################################################
    # checkMySQLConfiguration
    #
    # check MySQL configuration
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def checkMySQLConfiguration(self):

        if mysql.checkPasswordSilently(self.mysql_root_password) == 1:
            # password does not match
            if self.mysql_root_password !="":
                if mysql.checkPasswordSilently("") == 0:
                    glib.printErrorMessage("Configuration detected unprotected DB (no password)")
                    glib.printErrorMessage("and a non-empty password specified in the configuration parameters.")
                    glib.printErrorMessage("Please set the password in the MySQL DB via:")
                    glib.printErrorMessage("  /usr/bin/mysqladmin -u root password 'yourPassword'")
                    glib.printErrorMessage("  /usr/bin/mysqladmin -u root -h your.host.name password 'yourPassword'")                        
                    glib.printErrorMessage("and/or correct the value in the parameter 'rgma.server.mysql_root_password'")
                    return 1
                else:
                    glib.printErrorMessage("MySQL password incorrect. Please check the parameter 'rgma.server.mysql_root_password'")
                    return 1
        return 0


    #################################################################
    # configureTomcatWebapp
    #
    # configure Tomcat web application
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configureTomcatWebapp(self):

        print "\nConfiguring TOMCAT  ...                                  "
        
        errorcode = 0
        
        # configure tomcat
        errorcode += tomcat.configure()

        # set 'maxThreads' parameter for connector
        errorcode += tomcat.setConnectorParameter('8443', "maxThreads", self.httpconnector_maxthreads)

        # set 'enableLookups' parameter for connector
        errorcode += tomcat.setConnectorParameter('8443', "enableLookups", self.httpconnector_enableLookups)

        # set 'maxPostSize' parameter for connector
        errorcode += tomcat.setConnectorParameter('8443', "maxPostSize", self.httpconnector_maxPostSize)

        # set parameters for unsecure port (if port is enabled)
        if self.allow_unsecure_port=="true":
            
            # set 'maxThreads' parameter for connector
            errorcode += tomcat.setConnectorParameter('8080', "maxThreads", self.httpconnector_maxthreads)

            # set 'enableLookups' parameter for connector
            errorcode += tomcat.setConnectorParameter('8080', "enableLookups", self.httpconnector_enableLookups)
            
            # set 'maxPostSize' parameter for connector
            errorcode += tomcat.setConnectorParameter('8080', "maxPostSize", self.httpconnector_maxPostSize)

        # set kernel version
        errorcode += tomcat.setTomcatConfEntry("LD_ASSUME_KERNEL", self.server_ldAssumeKernel)

        # overwrite CATALINA_OPTS with rgma specific settings
        errorcode += tomcat.setTomcatConfEntry("CATALINA_OPTS", '"%s"' % self.rgma_server_tomcat_catalina_opts)

        print "Configuring TOMCAT                                        ",                    
        if errorcode:
            glib.printFailedMessage()
            print
            return 1
        
        glib.printOkMessage()
                                                
        # ------------------------------------------------------------
        # deploy R-GMA web app
        # ------------------------------------------------------------

        print "\nDeploying R-GMA server web application in Tomcat ...      "
    
        errorcode = tomcat.deploy(self.rgma_webapp_path, '%s/share/webapps/%s' % (os.environ['GLITE_LOCATION'], self.rgma_war_name))

        if errorcode:
            print "Deploying R-GMA server web application in Tomcat          ",            
            glib.printFailedMessage()
            return 1
        else:
            print "Deploying R-GMA server web application in Tomcat          ",            
            glib.printOkMessage()      
            return 0


    #################################################################
    # configureRgmaServer
    #
    # configure R-GMA server
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configureRgmaServer(self):

        print "Setting up R-GMA databases                                ",
        
        # run server setup script
        if self.verbose == 0:
            errorcode = os.system('python %s/share/rgma/scripts/rgma-server-setup.py --schema=%s --registry=%s --browser=%s --silent' \
                % (os.environ['GLITE_LOCATION'], self.run_schema_service, self.run_registry_service, self.run_browser))/256
        else:
            errorcode = os.system('python %s/share/rgma/scripts/rgma-server-setup.py --schema=%s --registry=%s --browser=%s' \
                % (os.environ['GLITE_LOCATION'], self.run_schema_service, self.run_registry_service, self.run_browser))/256
                            
        if errorcode:
            glib.printFailedMessage()
            return 1
        
        glib.printOkMessage()
        
        # ------------------------------------------------------------
        # fill MySQL tables
        # ------------------------------------------------------------

        print "Checking existance of databases ...                       "

        # check if DBs exists already
        registry_conflict = 0
        schema_conflict = 0
        status = 0
        
        # check conflicts with existing DBs
        if (self.run_registry_service == "yes") and (mysql.existsDB("registry", self.mysql_root_password)==0):
            registry_conflict = 1
            if self.verbose:
                print
            glib.printWarningMessage("Registry database exists already.")
        if (self.run_schema_service == "yes") and (mysql.existsDB("schema", self.mysql_root_password)==0):
            schema_conflict = 1
            if self.verbose:
                print
            glib.printWarningMessage("Schema database exists already.")    
                
        # conflict
        if (registry_conflict or schema_conflict):
            
            # conflict but forced
            if (self.recreate_db=="true"):
                glib.printInfoMessage("Option specified to force recreation of database(s).")
                glib.printInfoMessage("Recreating database(s) ...")

                # fill MySQL tables
                status = 0
                if self.verbose == 0:
                    status_output = commands.getstatusoutput("mysql -u root --password='%s' < %s/etc/rgma-server/rgma_sql_conf.sql" \
                        % (self.mysql_root_password, os.environ['GLITE_LOCATION']))
                    status = status_output[0]/256
                    output = status_output[1]
                else:                        
                    status = os.system("mysql -u root --password='%s' < %s/etc/rgma-server/rgma_sql_conf.sql" \
                        % (self.mysql_root_password, os.environ['GLITE_LOCATION']))/256
                
                print "Loading of MySQL databases                                ",
                if status:
                    glib.printFailedMessage()
                    glib.printErrorMessage("Issuing 'mysql -u root --password=.... < .....' failed")
                    return 1
                glib.printOkMessage()
        
            # conflict and not forced
            else:
                glib.printWarningMessage("Database(s) will not be recreated!")
                glib.printWarningMessage("In order to force the recreation of the DBs, run the")
                glib.printWarningMessage("configuration script with the option '--recreate_db'")
            
        # no conflict
        else:
            
            print "Checking existance of databases                           ",
            glib.printOkMessage()
              
            # fill MySQL tables
            status = 0
            if self.verbose == 0:
                status_output = commands.getstatusoutput("mysql -u root --password='%s' < %s/etc/rgma-server/rgma_sql_conf.sql" \
                    % (self.mysql_root_password, os.environ['GLITE_LOCATION']))
                status = status_output[0]/256
                output = status_output[1]
            else:                        
                status = os.system("mysql -u root --password='%s' < %s/etc/rgma-server/rgma_sql_conf.sql" \
                    % (self.mysql_root_password, os.environ['GLITE_LOCATION']))/256
                
            print "Loading of MySQL databases                                ",
            if status:
                glib.printFailedMessage()
                glib.printErrorMessage("Issuing 'mysql -u root --password=.... < .....' failed")
                return 1
        
            glib.printOkMessage()
            return 0


    #################################################################
    # configureRgma
    #
    # configure RGMA
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configureRgma(self):
        
        registries_string = ""
        
        if self.verbose:
            print "\nConfiguring R-GMA ..."
        else:
            print "Configuring R-GMA                                         ",        
            
# the following line should be put back as soon as multiple registries are enabled again            
####
#        for item in params['rgma.registry.hostname']:
#
#            registries_string = registries_string + " --registry %s" % item                        
#            if self.verbose:
#                print "Setting up R-GMA to use registry server '%s'" % item
####

        # the following lines should be removed as soon as multiple registries are enabled again
        # start-new-codeblock
        registries_string = "--registry %s" % params['rgma.registry.hostname'][0]
        if self.verbose:
            print "Setting up R-GMA to use registry server '%s'" % params['rgma.registry.hostname'][0]
        # end-new-codeblock

        errorcode = 0
        errorcode = glib.export('RGMA_HOME', os.environ['GLITE_LOCATION'])

        if errorcode:
            if self.verbose:
                print "Configuring R-GMA                                         ",
            glib.printFailedMessage()
            glib.printErrorMessage("Could not export RGMA_HOME environment variable.")
            return 1
        
        # decide on security setting
        if self.rgma_secure_mode=="true":
            secure_setting="--secure=yes"
        else:
            secure_setting="--secure=no"
        
        # add server, schema and registry(ies)                                
        errorcode = 0
        if self.verbose == 0:
            errorcode += os.system('python %s/share/rgma/scripts/rgma-setup.py --server %s --schema %s %s %s --silent'  \
                % (os.environ['GLITE_LOCATION'], self.server_hostname, self.schema_hostname, registries_string, secure_setting))
        else:
            errorcode += os.system('python %s/share/rgma/scripts/rgma-setup.py --server %s --schema %s %s %s'  \
                % (os.environ['GLITE_LOCATION'], self.server_hostname, self.schema_hostname, registries_string, secure_setting))
        
        if errorcode:
            if self.verbose:
                print "Configuring R-GMA                                         ",                    
            glib.printFailedMessage()
            return 1

        if self.verbose:
            print "Configuring R-GMA                                         ",

        glib.printOkMessage()    
        return 0


    #################################################################
    # configureFlexibleArchiver
    #
    # configure flexible archiver
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configureFlexibleArchiver(self):    

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Configuring the R-GMA archiver"
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
        
        # ------------------------------------------------------------
        # Prepare glue archiver configuration file: set user, pasword and dbname
        # ------------------------------------------------------------
            
        try:                                        
            input = open(self.archiver_config_file, 'r')
            output = open(self.archiver_config_file + ".tmp", 'w')
            
            inlines = input.readlines()

            # exchange DB name, user and password and retention period
            for line in inlines:
                if line.startswith("DBName="):
                    output.write("DBName=jdbc:mysql://localhost/%s\n" % self.archiver_db_name)
                elif line.startswith("DBUsername="):
                    output.write("DBUsername=%s\n" % self.archiver_db_user)                    
                elif line.startswith("DBPassword="):
                    output.write("DBPassword=%s\n" % self.archiver_db_password)  
                elif line.startswith("HistoryRetentionPeriod="):
                    output.write("HistoryRetentionPeriod=%s\n" % self.archiver_historyRetentionPeriod)                      

                else:
                    output.write(line)  
                
            # close
            output.close()        
            input.close()
                
            # rename file
            os.rename(self.archiver_config_file + ".tmp", self.archiver_config_file)
                
        except IOError, ex:
            print "Configuring R-GMA glue-archiver                           ",    
            glib.printFailedMessage()
            glib.printErrorMessage("\nFile '%s' is not writable" % self.archiver_config_file + ".tmp")
            return 1

        # ------------------------------------------------------------
        # Create sql file to create DB and pipe them into MySQL
        # ------------------------------------------------------------

        # check conflicts if DB exists already
        archiver_conflict = "false"
        errorcode = 0
        if (mysql.existsDB(self.archiver_db_name, self.mysql_root_password)==0):
            glib.printWarningMessage("Archiver database exists already.")                
            if (self.recreate_db=="true"):
                glib.printInfoMessage("Option specified to force recreation of database.")
                glib.printInfoMessage("Recreating database ...")
            else:
                # conflict => no recreation
                archiver_conflict = "true"
                glib.printWarningMessage("Database will not be recreated!")
                glib.printWarningMessage("In order to force the recreation of the DBs, run the")
                glib.printWarningMessage("configuration script with the option '--recreate_db'")                                    
                    
        # recreate if no conflict or forced recreation
        if (archiver_conflict=="false"):
            errorcode = os.system("%s/bin/rgma-flexible-archiver-db-setup --dbname %s --user %s --passwd %s | mysql -u root --password=%s " % (os.environ['GLITE_LOCATION'],
                         self.archiver_db_name, self.archiver_db_user, self.archiver_db_password, self.mysql_root_password))                                
            
        if errorcode:
            print "Configuring R-GMA archiver                                ",
            glib.printFailedMessage()
            glib.printErrorMessage("Error running R-GMA flexible-archiver DB creation script")
            glib.printErrorMessage("'%s/bin/rgma-flexible-archiver-db-setup.py'" % os.environ['GLITE_LOCATION'])
            return 1
            
        print "Configuring R-GMA archiver                                ",
        glib.printOkMessage()
        return 0




    #################################################################
    # configureSitePublisher
    #
    # configure site publisher
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configureSitePublisher(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Configuring the R-GMA site publisher"
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"

        # ------------------------------------------------------------
        # write site-publisher configuration files
        # ------------------------------------------------------------

        print "Setting site publisher configuration file                 ",

        errorcode = 0
        
        try:                                        
            output = open(self.sitePublisher_configFile, 'w')
            output.write("site-name=%s\n"           % self.sitePublisher_siteId)
            output.write("sysAdminContact=%s\n"     % self.sitePublisher_sysAdminContact)
            output.write("userSupportContact=%s\n"  % self.sitePublisher_userSupportContact)
            output.write("siteSecurityContact=%s\n" % self.sitePublisher_securityContact)
            output.write("latitude=%s\n"            % self.sitePublisher_locationLatitude)
            output.write("longitude=%s\n"           % self.sitePublisher_locationLongitude)
            output.write("description=%s\n"         % self.sitePublisher_description)
            output.write("readableName=%s\n"        % self.sitePublisher_siteName)
            output.write("location=%s\n"            % self.sitePublisher_location)
            output.write("web=%s"                   % self.sitePublisher_webLocation)
            output.close()        
        except IOError, ex:
            if ex.strerror == 'Permission denied':
                glib.printFailedMessage()
                glib.printErrorMessage("\nSite-publisher configuration file '%s' is not writable" % self.sitePublisher_configFile)
                return 1
            else:
                glib.printFailedMessage()
                raise ex                                
                return 1

        glib.printOkMessage()        

        # ------------------------------------------------------------
        # correct bug for rgma-publish-site cronjob: source glite_setenv first in script 
        # ------------------------------------------------------------

        print "Setting JAVA_HOME for cronjob                             ",             
            
        inputfile = '/etc/cron.hourly/rgma-publish-site.cron'
        outputfile = '/etc/cron.hourly/rgma-publish-site.cron.tmp'
    
        # 0: header not processed
        # 1: header processing
        # 2: after header
        headerFlag = 0
            
        try:                                        
            input = open(inputfile, 'r')
            output = open(outputfile, 'w')
                
            inlines = input.readlines()
            for line in inlines:
                if line.startswith("#"):
                    output.write(line)  # normal comment line
                    if headerFlag == 0: # header not started before
                        headerFlag = 1  # header processing
                else:
                    if headerFlag == 1: # header processing before
                        # check if line does not exist yet (bug 14837)
                        if line != '. %s\n' % params['installer.export.filename']:
                            output.write('. %s\n' % params['installer.export.filename'])
                        output.write(line)
                        headerFlag = 2
                    else:
                        output.write(line)                            

            output.close()        
            input.close()
            os.rename(outputfile, inputfile)
            os.chmod(inputfile,0755)
        except IOError, ex:
            glib.printFailedMessage()
            glib.printErrorMessage("\nFile '%s' is not writable" % outputfile)
            return 1
        
        # Touch cron spool directory to cause reloading of the crontabs
        os.system("/bin/touch /var/spool/cron")

        glib.printOkMessage()
        return 0      


    #################################################################
    # configureRgmaServerSecurity
    #
    # configure R-GMA server security
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configureRgmaServerSecurity(self):
        
        print "Configuring R-GMA server security                         ",

        tomcat_user_home_dir = pwd.getpwnam(self.tomcat_user_name)[5]        

        try:
            file = open(self.server_security_config_file, 'w')
            file.write('sslCertFile=%s/%s/hostcert.pem\n' % (tomcat_user_home_dir, self.user_certificates_path))
            file.write('sslKey=%s/%s/hostkey.pem\n' % (tomcat_user_home_dir, self.user_certificates_path))
            file.write('crlEnabled=true\n')
            file.write('crlFiles=%s/*.r0\n' % self.ca_certificates_dir)
            file.write('sslCAFiles=%s/*.0\n' % self.ca_certificates_dir)
            file.close()
        except IOError, ex:
            if ex.strerror == 'Permission denied':
                glib.printFailedMessage()
                glib.printErrorMessage("File '%s' is not writable" % self.server_security_config_file)
                return 1
            else:
                glib.printFailedMessage()
                raise ex
                return 1

        glib.printOkMessage()
        return 0


    #################################################################
    # configureRgmaClient
    #
    # configure R-GMA server security
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configureRgmaClient(self):

        # ------------------------------------------------------------
        # Create R-GMA user and group (site-publisher/archiver run in this account)
        # ------------------------------------------------------------

        if self.verbose == 0:
            print "Creating R-GMA user and group                             ",
        else:
            print "\nCreating R-GMA user and group ...                         "
        
        if glib.add_user(self.rgma_user_name,self.rgma_user_group) == -1:
            glib.printFailedMessage()
            return 1

        if self.verbose == 0:            
            glib.printOkMessage()
        else:
            print "Creating R-GMA user and group                             ",
            glib.printOkMessage()
            
        # ------------------------------------------------------------
        # Updating host key and certificate
        # ------------------------------------------------------------

        print "Updating hostkeys and certificates for R-GMA client       ",

        self.rgma_user_home_dir = pwd.getpwnam(self.rgma_user_name)[5]

        self.user_cert_filename = "hostcert.pem"
        self.user_key_filename  = "hostkey.pem"

        # create dir user_certificates_path if it does not exist
        if not os.path.exists("%s/%s" % (self.rgma_user_home_dir, self.user_certificates_path)):
            os.mkdir("%s/%s" % (self.rgma_user_home_dir, self.user_certificates_path),0755)

        # copy key and cert file        
        errorcode = 0
        errorcode += os.system("cp %s %s/%s/%s" % \
            (self.host_key_file, self.rgma_user_home_dir, self.user_certificates_path, self.user_key_filename))/256
        errorcode += os.system("cp %s %s/%s/%s" % \
            (self.host_certificate_file, self.rgma_user_home_dir, self.user_certificates_path, self.user_cert_filename))/256
        if errorcode:
            glib.printFailedMessage()
            glib.printErrorMessage("Could not copy certificate and key for user '%s'" % self.rgma_flexible_archiver_user_id)
            glib.printErrorMessage("to files '%s/%s/%s' and '%s/%s/%s'" % \
                (self.rgma_user_home_dir, self.user_certificates_path, self.user_cert_filename, \
                  self.rgma_user_home_dir, self.user_certificates_path, self.user_key_filename))
            return 1
    
        # change owner of key and cert file        
        errorcode = 0
        errorcode += os.system("chown %s:%s %s/%s/%s" % \
            (self.rgma_user_name, self.rgma_user_group, \
             self.rgma_user_home_dir, self.user_certificates_path, self.user_key_filename))/256
        errorcode += os.system("chown %s:%s %s/%s/%s" % \
            (self.rgma_user_name, self.rgma_user_group, \
             self.rgma_user_home_dir, self.user_certificates_path, self.user_cert_filename))/256
    
        if errorcode:
            glib.printFailedMessage()
            glib.printErrorMessage("Could not change access rights of certificate and key for user '%s:%s'" % \
                (self.rgma_user_name, self.rgma_user_group))
            glib.printErrorMessage("for file '%s/%s/hostkey.pem' and '%s/%s/hostcert.pem'" % \
                (self.rgma_user_home_dir, self.user_certificates_path))
            return 1

        glib.printOkMessage()                          
        return 0


    #################################################################
    # configureRgmaClientSecurity
    #
    # configure R-GMA server security
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configureRgmaClientSecurity(self):

        print "Configuring R-GMA client security                         ",

        path = "%s/etc/rgma/ClientAuthentication.props" % os.environ['GLITE_LOCATION']

        try:
            file = open(path, 'w')
            file.write('sslCertFile=%s/%s/%s\n' % (self.rgma_user_home_dir, self.user_certificates_path, self.user_cert_filename))
            file.write('sslKey=%s/%s/%s\n' % (self.rgma_user_home_dir, self.user_certificates_path, self.user_key_filename))
            file.write('crlFiles=%s/*.r0\n' % self.ca_certificates_dir)
            file.write('sslCAFiles=%s/*.0\n' % self.ca_certificates_dir)
            file.write('#gridProxyFile=/tmp/grid-user-proxy.pem\n')
            file.close()
        except IOError, ex:
            if ex.strerror == 'Permission denied':
                glib.printFailedMessage()
                glib.printErrorMessage("File '%s' is not writable" % path)
                return 1
            else:
                glib.printFailedMessage()
                raise ex
                return 1

        glib.printOkMessage()
        return 0



    #################################################################
    # start
    #
    # start rgma server
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def start(self):    

        # for the moment we do a restart if service is already running
        self.stop()   

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Starting the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"

        # ------------------------------------------------------------
        # Start MySQL (if not yet running)
        # ------------------------------------------------------------

        mysql.start()
   
        # ------------------------------------------------------------
        # start tomcat
        # ------------------------------------------------------------
        
        # for the moment we do a restart if service is already running
        errorcode = tomcat.conditionalStart()
        if errorcode:
            print "Starting R-GMA server                                     ",            
            glib.printFailedMessage()
            return 1
            
        print "Starting R-GMA server                                     ",        
        glib.printOkMessage()
        
        # give tomcat time to get up to full speed :)
        print "Waiting for tomcat to fully startup ...                   "
        time.sleep(20)

        # rgma run archiver (user)
        if params.has_key('rgma.server.run_archiver'):
            
            # ensure backwards compatability:
            if params['rgma.server.run_archiver'] == "yes":
                glib.printWarningMessage("Parameter 'rgma.server.run_archiver' is set to yes.")
                glib.printWarningMessage("Parameter has changed to a boolean parameter (true|false) and is no longer (yes|no).") 
                glib.printWarningMessage("Please change the parameter to 'true|false'.")       
                glib.printWarningMessage("To ensure backwards compatability, the parameter is taken as 'true' for the configuration.")
                
                self.run_archiver = "yes"
            
            # ensure backwards compatability:    
            elif params['rgma.server.run_archiver'] == "no":
                glib.printWarningMessage("Parameter 'rgma.server.run_archiver' is set to no.")
                glib.printWarningMessage("Parameter has changed to a boolean parameter (true|false) and is no longer (yes|no).") 
                glib.printWarningMessage("Please change the parameter to 'true|false'.")       
                glib.printWarningMessage("To ensure backwards compatability, the parameter is taken as 'false' for the configuration.")
                
                self.run_archiver = "no"
            
            elif params['rgma.server.run_archiver'] == "true":
                
                self.run_archiver = "yes"
                
            elif params['rgma.server.run_archiver'] == "false":
                
                self.run_archiver = "no"
                
            else:
                glib.printErrorMessage("Parameter 'rgma.server.run_archiver' specified incorrectly.")
                glib.printErrorMessage("Must be 'true|false' while '%s' has been specified." % params['rgma.server.run_archiver'])
                glib.printErrorMessage("Correct error and re-run configuration.")
                return 1
                
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.server.run_archiver\" not found.")
            return 1

        # rgma run site-publisher (user)
        if params.has_key('rgma.server.run_site-publisher'):
            
            # ensure backwards compatability:
            if params['rgma.server.run_site-publisher'] == "yes":
                glib.printWarningMessage("Parameter 'rgma.server.run_site-publisher' is set to yes.")
                glib.printWarningMessage("Parameter has changed to a boolean parameter (true|false) and is no longer (yes|no).") 
                glib.printWarningMessage("Please change the parameter to 'true|false'.")       
                glib.printWarningMessage("To ensure backwards compatability, the parameter is taken as 'true' for the configuration.")
                
                self.run_site_publisher = "yes"
            
            # ensure backwards compatability:    
            elif params['rgma.server.run_site-publisher'] == "no":
                glib.printWarningMessage("Parameter 'rgma.server.run_site-publisher' is set to no.")
                glib.printWarningMessage("Parameter has changed to a boolean parameter (true|false) and is no longer (yes|no).") 
                glib.printWarningMessage("Please change the parameter to 'true|false'.")       
                glib.printWarningMessage("To ensure backwards compatability, the parameter is taken as 'false' for the configuration.")
                
                self.run_site_publisher = "no"
            
            elif params['rgma.server.run_site-publisher'] == "true":
                
                self.run_site_publisher = "yes"
                
            elif params['rgma.server.run_site-publisher'] == "false":
                
                self.run_site_publisher = "no"
                
            else:
                glib.printErrorMessage("Parameter 'rgma.server.run_site-publisher' specified incorrectly.")
                glib.printErrorMessage("Must be 'true|false' while '%s' has been specified." % params['rgma.server.run_site-publisher'])
                glib.printErrorMessage("Correct error and re-run configuration.")
                return 1
                
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.server.run_site-publisher\" not found.")
            return 1

            
        if  self.run_archiver=="yes":

            # ------------------------------------------------------------
            # start glue-archiver
            # ------------------------------------------------------------
                        
            # check status of glue-archiver
            status_output = commands.getstatusoutput("%s/etc/init.d/rgma-glue-archiver status" % os.environ['GLITE_LOCATION'])
            status = status_output[0]/256
            output = status_output[1]

            if status==0:
            
                # glue-archiver is running
                glib.printInfoMessage("R-GMA glue-archiver is already running. Stopping and restarting glue-archiver ...")
            
                # stop glue archiver first
                errorcode = os.system("%s/etc/init.d/rgma-glue-archiver stop" % os.environ['GLITE_LOCATION'])/256
                if errorcode:
                    glib.printErrorMessage("Stopping R-GMA glue-archiver failed.")
                    return 1

            else:
                if status !=1:
                    glib.printWarningMessage("R-GMA glue-archiver is in unknown status. Trying to start glue-archiver ...")
                    glib.printWarningMessage("Status script returned: %s" % output)
                                    
            flexible_archiver_lock_file = "/var/lock/subsys/rgma-glue-archiver"
            flexible_archiver_id_file = "%s/var/rgma-glue-archiver/rgma-glue-archiver.id" % os.environ['GLITE_LOCATION']
            lock_file = 0
            id_file = 0
            if os.path.exists(flexible_archiver_lock_file):
                lock_file = 1
                glib.printWarningMessage("R-GMA glue archiver lock file exists at")
                glib.printWarningMessage("   %s" % flexible_archiver_lock_file)
            if os.path.exists(flexible_archiver_id_file):
                id_file = 1
                glib.printWarningMessage("R-GMA glue archiver id file exists at")
                glib.printWarningMessage("   %s" % flexible_archiver_id_file)
            if id_file ==1 or lock_file ==1:
                glib.printWarningMessage("It looks like the application has not been stopped cleanly.")                
                glib.printWarningMessage("Please have a look at the log files.")
                glib.printWarningMessage("Lock/Id file will be removed to start glue archiver.")
                os.system("rm %s" % flexible_archiver_lock_file)

            print "Starting R-GMA glue-archiver. Please be patient this may take several minutes ..."
            print "Starting R-GMA glue-archiver                              ",
                        
            # start flexible archiver
            status_output = commands.getstatusoutput("%s/etc/init.d/rgma-glue-archiver start" % os.environ['GLITE_LOCATION'])
            status = status_output[0]/256
            output = status_output[1]          
            if status !=0:
                glib.printFailedMessage()
                return 1

            glib.printOkMessage()
                        
        if self.run_site_publisher=="yes":
            
            # ------------------------------------------------------------
            # start site-publisher
            # ------------------------------------------------------------
                        
            # check status of site-publisher
            status_output = commands.getstatusoutput("%s/etc/init.d/rgma-publish-site status" % os.environ['GLITE_LOCATION'])
            status = status_output[0]/256
            output = status_output[1]
            
            if status==0:
            
                # site-publisher is running
                glib.printInfoMessage("R-GMA site-publisher is already running. Stopping and restarting site-publisher ...")
            
                # stop site-publisher first
                errorcode = os.system("%s/etc/init.d/rgma-publish-site stop" % os.environ['GLITE_LOCATION'])/256
                if errorcode:
                    glib.printErrorMessage("Stopping R-GMA site-publisher failed.")
                    return 1

            else:
                if status !=1:
                    glib.printWarningMessage("R-GMA site-publisher is in unknown status. Trying to start R-GMA site-publisher ...")
                    glib.printWarningMessage("Status script returned: %s" % output)

            print "Starting R-GMA site-publisher                             ",
                        
            # start site-publisher            
            errorcode = os.system("%s/etc/init.d/rgma-publish-site start" % os.environ['GLITE_LOCATION'])/256
            if errorcode:
                glib.printFailedMessage()
                return 1

            glib.printOkMessage()
                        
        #-------------------------------------------------------------------
        # start R-GMA servicetool
        #-------------------------------------------------------------------
        errocode = rgmaServicetool.start()
        if errorcode:
            return 1        
        
        print "\nIn order to use the R-GMA server you need to ensure that the"
        print "ports 8080 (for the unsecure version) and/or 8443 (for the secure" 
        print "version) are open!"
        print 
        print "To check if your server is running correctly you can run "
        print "    %s/etc/config/scripts/glite-rgma-server-config.py --status" % os.environ['GLITE_LOCATION']

        return 0

        
    #################################################################
    # stop
    #
    # stop rgma server
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def stop(self):    

        #-------------------------------------------------------------------
        # stop R-GMA servicetool
        #-------------------------------------------------------------------
        errorcode = rgmaServicetool.stop()
        if errorcode:
            return 1        
    
        # ------------------------------------------------------------
        # we stop site-publisher and glue-archiver independent 
        # of their setting in the configuration file
        # ------------------------------------------------------------

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Stopping the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
            
        # ------------------------------------------------------------
        # stop site-publisher if running
        # ------------------------------------------------------------
        
        # check status of site-publisher
        status_output = commands.getstatusoutput("%s/etc/init.d/rgma-publish-site status" % os.environ['GLITE_LOCATION'])
        status = status_output[0]/256
        output = status_output[1]
        running = 1
            
        if status==1:
            
            # site-publisher is not running
            glib.printInfoMessage("R-GMA site-publisher is not running.")
            running = 0
        
        else:
            if status !=0:
                glib.printWarningMessage("R-GMA site-publisher is in unknown status. Trying to stop site-publisher ...")
                glib.printWarningMessage("Status script returned: %s" % output)

        if running == 1:
            print "Stopping R-GMA site-publisher                             ",
            
            # stop site-publisher            
            errorcode = os.system("%s/etc/init.d/rgma-publish-site stop" % os.environ['GLITE_LOCATION'])/256
            if errorcode:
                glib.printFailedMessage()
                return 1

            glib.printOkMessage()        
                    
            
        # ------------------------------------------------------------
        # stop glue-archiver if running
        # ------------------------------------------------------------
        
        # check status of glue-archiver
        status_output = commands.getstatusoutput("%s/etc/init.d/rgma-glue-archiver status" % os.environ['GLITE_LOCATION'])
        status = status_output[0]/256
        output = status_output[1]
        running = 1
            
        if status==1:
            
            # glue-archiver is not running
            glib.printInfoMessage("R-GMA glue-archiver is not running.")
            running = 0
        
        else:
            if status !=0:       
                glib.printWarningMessage("R-GMA glue-archiver is in unknown status. Trying to stop glue-archiver ...")
                glib.printWarningMessage("Status script returned: %s" % output)

        if running == 1:
            print "Stopping R-GMA glue-archiver                              ",
                        
            # stop glue archiver            
            errorcode = os.system("%s/etc/init.d/rgma-glue-archiver stop" % os.environ['GLITE_LOCATION'])/256
            if errorcode:
                glib.printFailedMessage()
                return 1

            glib.printOkMessage()
        
        # ------------------------------------------------------------
        # stop tomcat
        # ------------------------------------------------------------
        
        errorcode = tomcat.stop()
        print "\nStopping R-GMA server                                     ",
        if errorcode:
            glib.printFailedMessage()
            return 1
        
        glib.printOkMessage()
            
        return 0

        
    #################################################################
    # status
    #
    # print status of R-GMA server and other components
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def status(self):    
        
        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Checking status of %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
 
        print "Please wait - this might take a while.\n"
                
        # ------------------------------------------------------------
        # read configuration parameters
        # ------------------------------------------------------------

        # rgma run archiver
        if params.has_key('rgma.server.run_archiver'):
            self.run_archiver = params['rgma.server.run_archiver']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.server.run_archiver\" not found.")
            return 1
            
        # rgma run site publisher
        if params.has_key('rgma.server.run_site-publisher'):
            self.run_site_publisher = params['rgma.server.run_site-publisher']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.server.run_site-publisher\" not found.")
            return 1
    
        # ------------------------------------------------------------
        # print status of R-GMA server
        # ------------------------------------------------------------

        errorcode = 0

        if verbose==0:
        
            status_output = commands.getstatusoutput("%s/bin/rgma-server-check" % os.environ['GLITE_LOCATION'])
            status = status_output[0]/256
            output = status_output[1]

            if status==1:
                print("R-GMA server is not running.")
                errorcode = 1
            else:
                if status==0:
                    print "R-GMA server is running."
                else:
                    # site-publisher is in unknown state
                    glib.printWarningMessage("R-GMA server is in unknown state.")
                    glib.printWarningMessage("Status script returned: %s" % output)
                    errorcode = 1
        
        else:
            returnvalue = os.system("%s/bin/rgma-server-check" % os.environ['GLITE_LOCATION'])
        
            if returnvalue:
                print "R-GMA server is not running or not running correctly."
                errorcode = 1
            else:
                print "R-GMA server is running."
            
        
        if  self.run_site_publisher=="yes":

            # ------------------------------------------------------------
            # print status of R-GMA site publisher
            # ------------------------------------------------------------
        
            status_output = commands.getstatusoutput("%s/etc/init.d/rgma-publish-site status" % os.environ['GLITE_LOCATION'])
            status = status_output[0]/256
            output = status_output[1]

            if status==1:
            
                # site-publisher is not running
                print("R-GMA site-publisher is not running.")
                errorcode = 1
        
            else:
                if status==0:
                
                    # site-publisher is running
                    print("R-GMA site-publisher is running.")
                
                else:
                    # site-publisher is in unknown state
                    glib.printWarningMessage("R-GMA site-publisher is in unknown state.")
                    glib.printWarningMessage("Status script returned: %s" % output)
                    errorcode = 1

        if  self.run_archiver=="yes":
        
            # ------------------------------------------------------------
            # print status of R-GMA glue archiver
            # ------------------------------------------------------------        

            # check status of glue-archiver
            status_output = commands.getstatusoutput("%s/etc/init.d/rgma-glue-archiver status" % os.environ['GLITE_LOCATION'])
            status = status_output[0]/256
            output = status_output[1]
            
            if status==1:
            
                # glue-archiver is not running
                print("R-GMA glue-archiver is not running.")
                errorcode = 1
        
            else:
                if status == 0:
                    print("R-GMA glue-archiver is running.")            
                else:
                    glib.printWarningMessage("R-GMA glue-archiver is in unknown state.")
                    glib.printWarningMessage("Status script returned: %s" % output)
                    errorcode = 1
        
        # ------------------------------------------------------------
        # print status of R-GMA service publisher
        # ------------------------------------------------------------

        # check status of service publisher
        
        status_output = commands.getstatusoutput("%s/etc/config/scripts/glite-rgma-servicetool-config.py --status" % os.environ['GLITE_LOCATION'])
        status = status_output[0]/256
        output = status_output[1]
        
        if status==1:
            
            # glue-archiver is not running
            print("R-GMA service publisher is not running.")
            errorcode = 1
        
        else:
            if status == 0:
                print("R-GMA service publisher is running.")            
            else:
                glib.printWarningMessage("R-GMA service publisher is in unknown state.")
                glib.printWarningMessage("Status script returned: %s" % output)
                errorcode = 1
                
        return errorcode


    #################################################################
    # showConfiguration
    #
    # print major configuration
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def showConfiguration(self):
        
        self.copyright()
        self.showVersion()
        glib.print_params(params)

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "             Main settings of %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"

        print "  R-GMA SERVER"
        print "     --------------------------------------------------------------"
        if params.has_key('rgma.server.hostname'):
            print "     \033[34m[Location]\x1b[0m          %s " % params['rgma.server.hostname']
        else:
            glib.printErrorMessage("Parameter 'rgma.server.hostname' not found.")
        if params.has_key('allow.unsecure.port'):
            print "     \033[34m[AllowUnsecurePort]\x1b[0m %s " % params['allow.unsecure.port']
        else:
            glib.printErrorMessage("Parameter 'allow.unsecure.port' not found.")    
        print "     --------------------------------------------------------------"
        print
        
        # print schema settings
        print "  R-GMA SCHEMA SERVER"
        print "     --------------------------------------------------------------"
        if params.has_key('rgma.server.run_schema_service'):
            print "     \033[34m[Enabled on host]\x1b[0m   %s " % params['rgma.server.run_schema_service']
        else:
            glib.printErrorMessage("Parameter 'rgma.server.run_schema_service' not found.")
        if params.has_key('rgma.schema.hostname'):
            print "     \033[34m[Location]\x1b[0m          %s " % params['rgma.schema.hostname']
        else:
            glib.printErrorMessage("Parameter 'rgma.schema.hostname' not found.")        
        print "     --------------------------------------------------------------"
        print
        
        # print registry settings
        print "  R-GMA REGISTRY SERVER"        
        print "     --------------------------------------------------------------"
        if params.has_key('rgma.server.run_registry_service'):
            print "     \033[34m[Enabled on host]\x1b[0m   %s " % params['rgma.server.run_schema_service']
        else:
            glib.printErrorMessage("Parameter 'rgma.server.run_registry_service' not found.")        

# The following lines should be put in place as soon as multiple registries in enabled again
######
#        if params.has_key('rgma.registry.hostname'):
#            for item in params['rgma.registry.hostname']: 
#                print "     \033[34m[Location]\x1b[0m          %s " % item
#        else:
#            glib.printErrorMessage("Parameter 'rgma.registry.hostname' not found.")        
######

        # The following lines can be removed as soon as multiple registries are enabled again
        # start-new-block
        if params.has_key('rgma.registry.hostname'):
            print "     \033[34m[Location]\x1b[0m          %s " % params['rgma.registry.hostname'][0]
        else:
            glib.printErrorMessage("Parameter 'rgma.registry.hostname' not found.")    
        
        if params.has_key('rgma.registry.hostname'):
            if len(params['rgma.registry.hostname']) > 1:
                print
                glib.printWarningMessage("You specified more than one registry.")
                glib.printWarningMessage("At the moment only one registry is allowed.")
                glib.printWarningMessage("Configuration will use first specified registry '%s'" % params['rgma.registry.hostname'][0])
                glib.printWarningMessage("and ignore the others.")
                print        
        # end-new-block

        print "     --------------------------------------------------------------"
        print

        # print registry settings
        print "  R-GMA BROWSER"        
        print "     --------------------------------------------------------------"
        if params.has_key('rgma.server.run_browser'):
            print "     \033[34m[Enabled on host]\x1b[0m   %s " % params['rgma.server.run_browser']
        else:
            glib.printErrorMessage("Parameter 'rgma.server.run_browser' not found.")        
        print "     --------------------------------------------------------------"
        print        
        
        # print archiver settings
        print "  R-GMA ARCHIVER"
        print "     --------------------------------------------------------------"
        if params.has_key('rgma.server.run_archiver'):
            print "     \033[34m[Enabled on host]\x1b[0m   %s " % params['rgma.server.run_archiver']
        else:
            glib.printErrorMessage("Parameter 'rgma.server.run_archiver' not found.")                                     
        if params.has_key('rgma.archiver.db.name'):
            print "     \033[34m[DB Name]\x1b[0m           %s " % params['rgma.archiver.db.name']
        if params.has_key('rgma.archiver.db.user'):
            print "     \033[34m[DB User]\x1b[0m           %s " % params['rgma.archiver.db.user']
        if params.has_key('rgma.archiver.db.password'):
            print "     \033[34m[DB Password]\x1b[0m       %s " % params['rgma.archiver.db.password']
        if params.has_key('rgma.archiver.db.historyRetentionPeriod'):
            print "     \033[34m[RetentionPeriod]\x1b[0m   %s minutes" % params['rgma.archiver.db.historyRetentionPeriod']
            
        print "     --------------------------------------------------------------"
        print
        
        # print site-publisher
        print "  R-GMA SITE-PUBLISHER"
        print "     --------------------------------------------------------------"
        if params.has_key('rgma.server.run_site-publisher'):
            print "     \033[34m[Enabled on host]\x1b[0m   %s " % params['rgma.server.run_site-publisher']
     
            if params.has_key('rgma.site-publisher.siteId'):
                print "     \033[34m[Site ID]\x1b[0m           %s " % params['rgma.site-publisher.siteId']

            if params.has_key('rgma.site-publisher.siteName'):
                print "     \033[34m[Site name]\x1b[0m         %s " % params['rgma.site-publisher.siteName']

            if params.has_key('rgma.site-publisher.description'):
                print "     \033[34m[Site description]\x1b[0m  %s " % params['rgma.site-publisher.description']
                
            if params.has_key('rgma.site-publisher.location'):
                print "     \033[34m[Location]\x1b[0m          %s " % params['rgma.site-publisher.location']

            if params.has_key('rgma.site-publisher.location.latitude'):
                print "     \033[34m[Location (lat)]\x1b[0m    %s (decimal degrees)" % params['rgma.site-publisher.location.latitude']

            if params.has_key('rgma.site-publisher.location.longitude'):
                print "     \033[34m[Location (lon)]\x1b[0m    %s (decimal degrees)" % params['rgma.site-publisher.location.longitude']

            if params.has_key('rgma.site-publisher.web'):
                print "     \033[34m[Site web location]\x1b[0m %s " % params['rgma.site-publisher.web']
                
            if params.has_key('rgma.site-publisher.contact.system_administrator'):
                print "     \033[34m[Sys admin contact]\x1b[0m %s " % params['rgma.site-publisher.contact.system_administrator']
            
            if params.has_key('rgma.site-publisher.contact.user_support'):
                print "     \033[34m[Support contact]\x1b[0m   %s " % params['rgma.site-publisher.contact.user_support']

            if params.has_key('rgma.site-publisher.contact.site_security'):
                print "     \033[34m[Security contact]\x1b[0m  %s " % params['rgma.site-publisher.contact.site_security']
        else:
            glib.printErrorMessage("Parameter 'rgma.server.run_site-publisher' not found.")                                       

        print "     --------------------------------------------------------------"

        print
        rgmaServicetool.showServices()
        
        return 0
        
#################################################################
# loadDefaultParameters
#
# load default parameters
#
# params params list of parameters
#
# return 0 ok
#        1 error
#
#################################################################
def loadDefaultParameters(params):
    
    # R-GMA common default parameters
    params['rgma.user.name']    = "rgma"
    params['rgma.user.group']   = "rgma"
    params['rgma.secure.mode']  = "true"

    # R-GMA internal default parameters
    params['rgma.server.recreate_db'] = "false"        
        
    # R-GMA server default values
    params['rgma.server.httpconnector_maxthread']     = '1000'
    params['rgma.server.httpconnector_enableLookups'] = 'true'
    params['rgma.server.httpconnector_maxPostSize']   = '0'
    params['rgma.server.LD_ASSUME_KERNEL']            = "2.4.19"
    params['rgma.server.webapp_path']                 = "R-GMA"
    params['rgma.server.war_name']                    = "R-GMA.war"
    params['rgma.server.security.configurationFile']  = "${GLITE_LOCATION}/etc/rgma-server/ServletAuthentication.props"
    params['allow.unsecure.port']                     = 'false'
    params['rgma.server.tomcat.CATALINA_OPTS']        = '-Xmx${HALF_MEMORY_SIZE} -server -Dsun.net.client.defaultReadTimeout=60000 -Dsun.net.client.defaultConnectTimeout=10000'
    
    # R-GMA glue archiver default values        
    params['rgma.archiver.db.name']           = "arch0"
    params['rgma.archiver.db.user']           = "test"
    params['rgma.archiver.db.password']       = "info" 
    params['rgma.archiver.configurationFile'] = "${GLITE_LOCATION}/etc/rgma-glue-archiver/glue.config"
    params['rgma.archiver.db.historyRetentionPeriod'] = "90" # period in minutes
    
    # R-GMA site publisher default values
    params['rgma.site-publisher.siteId']            = "%s" % os.environ['HOSTNAME']
    params['rgma.site-publisher.configurationFile'] = "${GLITE_LOCATION}/etc/rgma-publish-site/site.props"

    return 0
        

#################################################################
# set_env
#
# setup environment
# 
#################################################################
def set_env():

    # gLite
    glib.export('GLITE_LOCATION');
    glib.export('GLITE_LOCATION_VAR');
    if not os.path.exists(os.environ['GLITE_LOCATION_VAR']):
        os.mkdir(os.environ['GLITE_LOCATION_VAR'],0755)
    glib.export('GLITE_LOCATION_LOG');
    if not os.path.exists(os.environ['GLITE_LOCATION_LOG']):
        os.mkdir(os.environ['GLITE_LOCATION_LOG'],0755)
    glib.export('GLITE_LOCATION_TMP');
    if not os.path.exists(os.environ['GLITE_LOCATION_TMP']):
        os.mkdir(os.environ['GLITE_LOCATION_TMP'],0755)

    glib.export('CATALINA_HOME')
    glib.export('JAVA_HOME')

    # bin and lib paths
    glib.addEnvPath("PATH","%s/externals/bin:%s/bin:%s/bin" % (os.environ['GLITE_LOCATION'],os.environ['GLITE_LOCATION'],os.environ['JAVA_HOME']))
    
    # Set environment
    glib.setUserEnv()    


#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # The script must be run as root
    if not os.geteuid()==0:
        print '"\nThis script must be run as root\n'
        sys.exit(1)

    # Get an instance of the library class
    glib = gLib()
    glib.verbose = 0
    verboseCLI = 0
    
    # ------------------------------------------------------------
    # Load parameters
    # ------------------------------------------------------------
    
    # Get an instance of the ConfigParams class
    params = ConfigParams()
    
    # load default parameters
    loadDefaultParameters(params)
    
    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version', 'configure', 'stop', 'start', 'status', 'siteconfig=', 'verbose', 'recreate_db'])
        for o, a in opts:
            
            if o == "--verbose":
                verboseCLI = 1
                glib.verbose = 1
                
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
                
    except getopt.GetoptError:
        pass
    
    # load rgma common and rgma server configuration parameters
    configFiles = ["%s/../glite-rgma-server.cfg.xml" % glib.getScriptPath(), 
                   "%s/../glite-rgma-common.cfg.xml" % glib.getScriptPath()]
        
    if glib.loadConfiguration(configFiles,params):
        glib.printErrorMessage("Problems while loading configuration file.")
        sys.exit(1)
    
    # ------------------------------------------------------------    
    # set verbosity
    # ------------------------------------------------------------    
    verbose = 0
    if verboseCLI:
        verbose = 1
    else:
        if params.has_key('glite.installer.verbose'):
            if params['glite.installer.verbose'] == "true":
                verbose = 1
    
    glib.verbose = verbose
    
    # ------------------------------------------------------------
    # Set up the environment
    # ------------------------------------------------------------
    set_env()

    # ------------------------------------------------------------          
    # Instantiate the service classes
    # ------------------------------------------------------------
    service = glite_rgma_server()
    service.verbose = verbose
    
    # ------------------------------------------------------------
    # Instantiate tomcat
    # ------------------------------------------------------------
    tomcat = Tomcat5(params)
    tomcat.verbose = verbose
    
    # ------------------------------------------------------------
    # Instantiate MySQL
    # ------------------------------------------------------------
    mysql = Mysql()
    mysql.verbose = verbose
    
    # ------------------------------------------------------------    
    # Instantiate the rgma servicetool class
    # ------------------------------------------------------------    
    rgmaServicetool = gliteRgmaServicetool()
    rgmaServicetool.verbose = verbose         
    
    # ------------------------------------------------------------
    # Command line opts if any
    # ------------------------------------------------------------
    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version', 'configure', 'stop', 'start', 'status', 'siteconfig=', 'verbose', 'recreate_db'])
    except getopt.GetoptError:
        service.usage(msg = "Unknown options(s)")
        sys.exit(1)

    # ------------------------------------------------------------
    # Check cli options
    # ------------------------------------------------------------    
    configure=0
    
    for o, a in opts:

        # show usage
        if o in ("-h", "--help"):            
            service.usage()
            sys.exit(0)

        # show version
        if o in ("-v", "--version"):            
            service.showVersion()
            sys.exit(0)

        # check configuration
        if o in ("-c", "--checkconf"):            
            service.showConfiguration()
            sys.exit(0)
        
        # force recreation of db if exist
        if o == "--recreate_db":
            params['rgma.server.recreate_db'] = "true"
        
        # start
        if o == "--start":            
            if service.start() == 0:
                print "\nThe %s was successfully started\n" % service.friendly_name
                sys.exit(0)
            else:
                print "An error occurred while starting the %s service" \
                    % service.friendly_name
                sys.exit(1)
    
        # stop
        if o == "--stop":            
            if service.stop() == 0:
                print "\nThe %s was successfully stopped\n" % service.friendly_name
                sys.exit(0)
            else:
                print "An error occurred while stopping the %s service" \
                    % service.friendly_name
                sys.exit(1)
    
        # status
        if o == "--status":            
            if service.status()==0:
                sys.exit(0)
            else:
                sys.exit(1)
                
        # configure
        if o == "--configure":            
            configure=1    
    
    #-------------------------------------------------------------------------------    
    # Configure
    #-------------------------------------------------------------------------------    
    if configure==1:
    
        # print banner 
        service.copyright()
        service.showVersion()

        # Print configuration parameters
        if verbose:
            glib.print_params(params)

        # stop service
        service.stop()

        # configure service
        if service.configure()==0:
            print "\n%s configuration successfully completed\n" % service.friendly_name
            print "To start the %s please run" % service.friendly_name
            print "    %s/etc/config/scripts/glite-rgma-server-config.py --start" % os.environ['GLITE_LOCATION']
            sys.exit(0)
        else:
            print "\nAn unrecoverable error occurred while configuring the %s service" \
                % service.friendly_name
            sys.exit(1)    
    
    else:            
        # ------------------------------------------------------------
        # No option given
        # ------------------------------------------------------------
        service.usage()
        sys.exit(0)            
        
